#include "raylib.h"

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <stdbool.h>

#if defined(PLATFORM_WEB)
    #include <emscripten/emscripten.h>
#endif

//----------------------------------------------------------------------------------
// Some Defines
//----------------------------------------------------------------------------------
#define PLAYER_MAX_LIFE         5
#define LINES_OF_BRICKS         6
#define BRICKS_PER_LINE        20

//----------------------------------------------------------------------------------
// Types and Structures Definition
//----------------------------------------------------------------------------------
typedef enum GameScreen { LOGO, TITLE, GAMEPLAY, ENDING } GameScreen;

typedef struct Player {
    Vector2 position;
    Vector2 size;
    int life;
} Player;

typedef struct Ball {
    Vector2 position;
    Vector2 speed;
    int radius;
    bool active;
} Ball;

typedef struct Brick {
    Vector2 position;
    bool active;
    int couleur;
} Brick;

//------------------------------------------------------------------------------------
// Global Variables Declaration
//------------------------------------------------------------------------------------
static const int screenWidth = 800;
static const int screenHeight = 550;

static int score= 0;
static int highScore = -50;
static int choixTheme=0;

static bool gameOver = false;
static bool pause = false;

static Player player = { 0 };
static Ball ball = { 0 };
static Brick brick[LINES_OF_BRICKS][BRICKS_PER_LINE] = { 0 };
static Vector2 brickSize = { 0 };

//------------------------------------------------------------------------------------
// Module Functions Declaration (local)
//------------------------------------------------------------------------------------
static void InitGame(void);         // Initialize game
static void UpdateGame(void);       // Update game (one frame)
static void DrawGame(void);         // Draw game (one frame)
static void UnloadGame(void);       // Unload game
static void UpdateDrawFrame(void);  // Update and Draw (one frame)

//------------------------------------------------------------------------------------
// Program main entry point
//------------------------------------------------------------------------------------
int main(void)
{
    // Initialization (Note windowTitle is unused on Android)
    //---------------------------------------------------------
    InitWindow(screenWidth, screenHeight, "Mini projet C Lea MESLIN : arkanoid");

    InitGame();

#if defined(PLATFORM_WEB)
    emscripten_set_main_loop(UpdateDrawFrame, 60, 1);
#else
    SetTargetFPS(60);
    //--------------------------------------------------------------------------------------
    
    // Main game loop
    while (!WindowShouldClose())    // Detect window close button or ESC key
    {
        // Update and Draw
        //----------------------------------------------------------------------------------
        UpdateDrawFrame();
        //----------------------------------------------------------------------------------
    }
#endif
    // De-Initialization
    //--------------------------------------------------------------------------------------
    UnloadGame();         // Unload loaded data (textures, sounds, models...)
    
    CloseWindow();        // Close window and OpenGL context
    //--------------------------------------------------------------------------------------

    return 0;
}

//------------------------------------------------------------------------------------
// Module Functions Definitions (local)
//------------------------------------------------------------------------------------

// Initialize game variables
void InitGame(void)
{
    brickSize = (Vector2){ GetScreenWidth()/BRICKS_PER_LINE, 40 };

    // Initialize player
    player.position = (Vector2){ screenWidth/2, screenHeight*7/8 };
    player.size = (Vector2){ screenWidth/10, 20 };
    player.life = PLAYER_MAX_LIFE;
    
    // Initialize ball
    ball.position = (Vector2){ screenWidth/2, screenHeight*7/8 - 30 };
    ball.speed = (Vector2){ 0, 0 };
    ball.radius = 7;
    ball.active = false;
    
    // Initialize bricks
    int initialDownPosition = 50;
    
    score=0;
    highScore=highScore;

    for (int i = 0; i < LINES_OF_BRICKS; i++)
    {
        for (int j = 0; j < BRICKS_PER_LINE; j++)
        {
            brick[i][j].position = (Vector2){ j*brickSize.x + brickSize.x/2, i*brickSize.y + initialDownPosition };
            brick[i][j].active = true;
            brick[i][j].couleur=0; //Valeur par défaut
        }
    }
    choixTheme = GetRandomValue(0,1);
    if (choixTheme==0){
        for (int i = 0; i < LINES_OF_BRICKS; i++)
        {
            for (int j = 0; j < BRICKS_PER_LINE; j++)
            {
                int random = GetRandomValue(0,3);
                switch(random){
                    case 0: {brick[i][j].couleur = 0;} break;
                    case 1: {brick[i][j].couleur = 1;} break;
                    case 2: {brick[i][j].couleur = 2;} break;
                    case 3: {brick[i][j].couleur = 3;} break;
                }
            }
        }
    }
    else {
        //Cas coeur 
        for (int i=0; i<6; i++){
            for (int j=2; j<9; j++){
                brick[i][j].couleur=2;
                brick[i][j+9].couleur=2;
            }
        }
        brick[4][5].couleur=3;
        brick[4][14].couleur=3;
        brick[3][4].couleur=3; 
        brick[3][13].couleur=3; 
        brick[3][6].couleur=3;
        brick[3][15].couleur=3;
        brick[2][3].couleur=3;
        brick[2][12].couleur=3;
        brick[2][5].couleur=3;
        brick[2][14].couleur=3;
        brick[2][7].couleur=3;
        brick[2][16].couleur=3;
        brick[1][3].couleur=3;
        brick[1][12].couleur=3;
        brick[1][4].couleur=3;
        brick[1][13].couleur=3;
        brick[1][6].couleur=3;
        brick[1][15].couleur=3;
        brick[1][7].couleur=3;
        brick[1][16].couleur=3;
        brick[2][6].couleur=1;
        brick[2][15].couleur=1;
        brick[2][4].couleur=1;
        brick[2][13].couleur=1;
        brick[3][5].couleur=1;
        brick[3][14].couleur=1;
    }
    
}

void briqueCouleurBleue(int i, int j){
    if ((brick[i][j].couleur== 1)){
        int k= i;
        int l= j;
        if (k==0){
            brick[k+1][l].active=false;
            brick[k][l-1].active=false; 
            brick[k][l+1].active=false;
        }
        else if (l==19 && k==0){
            brick[k+1][l].active=false;
            brick[k][l-1].active=false; 
        }   
        else if (l==0 && k==0){
            brick[k+1][l].active=false;
            brick[k][l+1].active=false; 
        }   
        else if (l==19 && k>0){
            brick[k+1][l].active=false;
            brick[k-1][l].active=false;
            brick[k][l-1].active=false;
        }
        else if (l==0 && k>0){
            brick[k+1][l].active=false;
            brick[k-1][l].active=false;
            brick[k][l+1].active=false;
        }
        else{
            brick[k+1][l].active=false;
            brick[k-1][l].active=false;
            brick[k][l-1].active=false;
            brick[k][l+1].active=false;
        }
    score+=4;
    }
}

void briqueCouleurNoire(int i, int j){
    if ((brick[i][j].couleur== 2)){
        brick[i][j].active=true;
        brick[i][j].couleur=0;
    }
}

void briqueCouleurViolette(int i, int j){
    if ((brick[i][j].couleur== 3)){
        ball.position.y=screenHeight*1/8;
        ball.speed.x+= -1/2;
    }
}

// Update game (one frame)
void UpdateGame(void)
{   
    
    if (!gameOver)
    {
        
        if (IsKeyPressed('P')) pause = !pause;

        if (!pause)
        {
            
             
            // Player movement logic
            if (IsKeyDown(KEY_LEFT)) player.position.x -= 5;
            if ((player.position.x - player.size.x/2) <= 0) player.position.x = player.size.x/2;
            if (IsKeyDown(KEY_RIGHT)) player.position.x += 5;
            if ((player.position.x + player.size.x/2) >= screenWidth) player.position.x = screenWidth - player.size.x/2;

            // Ball launching logic
            if (!ball.active)
            {
                if (IsKeyPressed(KEY_SPACE))
                {
                    ball.active = true;
                    ball.speed = (Vector2){ 0, -5 };
                }
            }
            
            // Ball movement logic
            if (ball.active)
            {
                ball.position.x += ball.speed.x;
                ball.position.y += ball.speed.y;
            }
            else
            {
                ball.position = (Vector2){ player.position.x, screenHeight*7/8 - 30 };
            }

            // Collision logic: ball vs walls 
            if (((ball.position.x + ball.radius) >= screenWidth) || ((ball.position.x - ball.radius) <= 0)) ball.speed.x *= -1;
            if ((ball.position.y - ball.radius) <= 0) ball.speed.y *= -1;
            if ((ball.position.y + ball.radius) >= screenHeight)
            {
                ball.speed = (Vector2){ 0, 0 };
                ball.active = false;

                player.life--;
                score-=10;
            }

            // Collision logic: ball vs player
            if (CheckCollisionCircleRec(ball.position, ball.radius,
                (Rectangle){ player.position.x - player.size.x/2, player.position.y - player.size.y/2, player.size.x, player.size.y}))
            {
                if (ball.speed.y > 0)
                {
                    ball.speed.y *= -1;
                    ball.speed.x = (ball.position.x - player.position.x)/(player.size.x/2)*5;
                }
            }

            // Collision logic: ball vs bricks
            for (int i = 0; i < LINES_OF_BRICKS; i++)
            {
                for (int j = 0; j < BRICKS_PER_LINE; j++)
                {
                    if (brick[i][j].active)
                    {
                        // Hit below
                        if (((ball.position.y - ball.radius) <= (brick[i][j].position.y + brickSize.y/2)) &&
                            ((ball.position.y - ball.radius) > (brick[i][j].position.y + brickSize.y/2 + ball.speed.y)) &&
                            ((fabs(ball.position.x - brick[i][j].position.x)) < (brickSize.x/2 + ball.radius*2/3)) && (ball.speed.y < 0))
                        {
                            brick[i][j].active = false;
                            ball.speed.y *= -1;
                            briqueCouleurBleue(i,j);
                            briqueCouleurNoire(i,j);
                            briqueCouleurViolette(i,j);
                            score +=1;
                        }
                        // Hit above
                        else if (((ball.position.y + ball.radius) >= (brick[i][j].position.y - brickSize.y/2)) &&
                                ((ball.position.y + ball.radius) < (brick[i][j].position.y - brickSize.y/2 + ball.speed.y)) &&
                                ((fabs(ball.position.x - brick[i][j].position.x)) < (brickSize.x/2 + ball.radius*2/3)) && (ball.speed.y > 0))
                        {
                            brick[i][j].active = false;
                            ball.speed.y *= -1;
                            briqueCouleurBleue(i,j);
                            briqueCouleurNoire(i,j);
                            briqueCouleurViolette(i,j);
                            score +=1;
                        }
                        // Hit left
                        else if (((ball.position.x + ball.radius) >= (brick[i][j].position.x - brickSize.x/2)) &&
                                ((ball.position.x + ball.radius) < (brick[i][j].position.x - brickSize.x/2 + ball.speed.x)) &&
                                ((fabs(ball.position.y - brick[i][j].position.y)) < (brickSize.y/2 + ball.radius*2/3)) && (ball.speed.x > 0))
                        {
                            brick[i][j].active = false;
                            ball.speed.x *= -1;
                            briqueCouleurBleue(i,j);
                            briqueCouleurNoire(i,j);
                            briqueCouleurViolette(i,j);
                            score +=1;
                        }
                        // Hit right
                        else if (((ball.position.x - ball.radius) <= (brick[i][j].position.x + brickSize.x/2)) &&
                                ((ball.position.x - ball.radius) > (brick[i][j].position.x + brickSize.x/2 + ball.speed.x)) &&
                                ((fabs(ball.position.y - brick[i][j].position.y)) < (brickSize.y/2 + ball.radius*2/3)) && (ball.speed.x < 0))
                        {
                            brick[i][j].active = false;
                            ball.speed.x *= -1;
                            briqueCouleurBleue(i,j);
                            briqueCouleurNoire(i,j);
                            briqueCouleurViolette(i,j);
                            score +=1;
                        }
                    }
                }
            }
            
            // Game over logic
            if (player.life <= 0){
                gameOver = true;
                int scoreFinPartie= score;
                if (scoreFinPartie > highScore) highScore = scoreFinPartie;
            }
            else
            {
                gameOver = true;
                for (int i = 0; i < LINES_OF_BRICKS; i++)
                {
                    for (int j = 0; j < BRICKS_PER_LINE; j++)
                    {
                        if (brick[i][j].active) gameOver = false;
                    }
                }
                if (gameOver){
                    int scoreFinPartie= score;
                    if (abs(scoreFinPartie) > highScore) highScore = scoreFinPartie;
                }
            }
        }
       
    }
    else
    {
        if (IsKeyPressed(KEY_ENTER))
        {
            InitGame();
            gameOver = false;
        }
    }
    
}

// Draw game (one frame)
void DrawGame(void)
{
    BeginDrawing();

        ClearBackground(RAYWHITE);

        if (!gameOver)
        {
            // Draw player bar
            DrawRectangle(player.position.x - player.size.x/2, player.position.y - player.size.y/2, player.size.x, player.size.y, BLACK);

            // Draw player lives
            for (int i = 0; i < player.life; i++) DrawRectangle(20 + 40*i, screenHeight - 30, 35, 10, LIGHTGRAY);
            
            DrawText(TextFormat("SCORE : %i", score),screenWidth -120 , screenHeight -30 , 15, BLACK);
            DrawText(TextFormat("HIGH SCORE : %i", highScore),screenWidth -160 , 10 , 15, BLACK);
            
            // Draw ball
            DrawCircleV(ball.position, ball.radius, MAROON);
            
            // Draw bricks
            for (int i = 0; i < LINES_OF_BRICKS; i++)
            {
                for (int j = 0; j < BRICKS_PER_LINE; j++)
                {
                    if (brick[i][j].active)
                    {
                        if (brick[i][j].couleur== 1) DrawRectangle (brick[i][j].position.x - brickSize.x/2, brick[i][j].position.y - brickSize.y/2, brickSize.x, brickSize.y, BLUE);
                        else if (brick[i][j].couleur== 2) DrawRectangle (brick[i][j].position.x - brickSize.x/2, brick[i][j].position.y - brickSize.y/2, brickSize.x, brickSize.y, BLACK);
                        else if (brick[i][j].couleur== 3) DrawRectangle (brick[i][j].position.x - brickSize.x/2, brick[i][j].position.y - brickSize.y/2, brickSize.x, brickSize.y, PURPLE);
                        else if ((i + j) % 2 == 0) DrawRectangle(brick[i][j].position.x - brickSize.x/2, brick[i][j].position.y - brickSize.y/2, brickSize.x, brickSize.y, GRAY);
                        else DrawRectangle(brick[i][j].position.x - brickSize.x/2, brick[i][j].position.y - brickSize.y/2, brickSize.x, brickSize.y, DARKGRAY);
                    }
                }
            }
            
            if (pause) DrawText("GAME PAUSED", screenWidth/2 - MeasureText("GAME PAUSED", 40)/2, screenHeight/2 - 40, 40, GRAY);
        }
        else {
            if (player.life ==0){
                ClearBackground(BLACK);
                DrawText("You'll do better next time :)", GetScreenWidth()/2 -MeasureText("You'll do better next time",20)/2, GetScreenHeight()/2 -90, 20, WHITE);
                DrawText("Press [Enter] to play again", GetScreenWidth()/2 -MeasureText("Play [Enter] to play again",20)/2, GetScreenHeight()/2 -45, 20, WHITE);
                DrawText(TextFormat("Your score : %i",score), GetScreenWidth()/2 -MeasureText("Your score : 25",25)/2, GetScreenHeight()/2 , 25, WHITE);
            }
            else {
                ClearBackground(BLACK);
                DrawText("Congrats you won !!", GetScreenWidth()/2 -MeasureText("Congrats you won !!",20)/2, GetScreenHeight()/2 -90, 20, WHITE);
                DrawText("Press [Enter] to play again", GetScreenWidth()/2 -MeasureText("Play [Enter] to play again",20)/2, GetScreenHeight()/2 -45, 20, WHITE);
                DrawText(TextFormat("Your score : %i",score), GetScreenWidth()/2 -MeasureText("Your score : 25",25)/2, GetScreenHeight()/2 , 25, WHITE);
            }
        }
    EndDrawing();
}

// Unload game variables
void UnloadGame(void)
{
    // TODO: Unload all dynamic loaded data (textures, sounds, models...)
}

// Update and Draw (one frame)
void UpdateDrawFrame(void)
{
    UpdateGame();
    DrawGame();
}